@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 import argparse
-import csv
 import os
 import re
+import pandas as pd
 from decimal import *
 
 
@@ -39,26 +39,11 @@ checkm_coverage
 """
 
 
-def flagstats_summary(flagstats, out_file, method):
+def flagstats_summary(flagstats, method, **kwargs):
     """
     get alignment rate from sorted bam file
-    samtools -flagstat --threads 8 sample.sort.bam
+    samtools flagstat --threads 8 sample.sort.bam
     """
-    headers = [
-        "sample_id",
-        "total_num",
-        "read_1_num",
-        "read_2_num",
-        "mapping_type",
-        "mapped_num",
-        "mapped_rate",
-        "paired_num",
-        "paired_rate",
-        "singletons_num",
-        "singletons_rate",
-        "mate_mapped_num",
-        "mate_mapped_num_mapQge5",
-    ]
     mapping_info = []
     getcontext().prec = 8
 
@@ -73,15 +58,34 @@ def flagstats_summary(flagstats, out_file, method):
             info = {}
             info["sample_id"] = os.path.basename(flagstat_file.strip()).split(".")[0]
             stat_list = open(flagstat_file.strip(), "r").readlines()
+
             info["total_num"] = stat_list[0].split(" ")[0]
-            info["read_1_num"] = stat_list[6].split(" ")[0]
-            info["read_2_num"] = stat_list[7].split(" ")[0]
 
-            mapped = re.split(r"\(|\s+", stat_list[4])
-            info["mapped_num"] = mapped[0]
-            info["mapped_rate"] = Decimal(mapped[5].rstrip("%")) / Decimal(100)
+            if len(stat_list) == 13:
+                info["read_1_num"] = stat_list[6].split(" ")[0]
+                info["read_2_num"] = stat_list[7].split(" ")[0]
 
-            paired = re.split(r"\(|\s+", stat_list[8])
+                mapped = re.split(r"\(|\s+", stat_list[4])
+                info["mapped_num"] = mapped[0]
+                info["mapped_rate"] = Decimal(mapped[5].rstrip("%")) / Decimal(100)
+            
+                #primary_mapped = re.split(r"\(|\s+", stat_list[5])
+                #info["primary_mapped_num"] = primary_mapped[0]
+                #info["primary_mapped_rate"] = Decimal(primary_mapped[6].rstrip("%")) / Decimal(100)
+
+            elif len(stat_list) == 16:
+                info["read_1_num"] = stat_list[9].split(" ")[0]
+                info["read_2_num"] = stat_list[10].split(" ")[0]
+
+                mapped = re.split(r"\(|\s+", stat_list[6])
+                info["mapped_num"] = mapped[0]
+                info["mapped_rate"] = Decimal(mapped[5].rstrip("%")) / Decimal(100)
+            
+                primary_mapped = re.split(r"\(|\s+", stat_list[7])
+                info["primary_mapped_num"] = primary_mapped[0]
+                info["primary_mapped_rate"] = Decimal(primary_mapped[6].rstrip("%")) / Decimal(100)
+ 
+            paired = re.split(r"\(|\s+", stat_list[-5])
             info["paired_num"] = paired[0]
             paired_rate = paired[6].rstrip("%")
             if paired_rate != "N/A":
@@ -103,10 +107,10 @@ def flagstats_summary(flagstats, out_file, method):
             info["mate_mapped_num_mapQge5"] = re.split(r"\(|\s+", stat_list[-1])[0]
             mapping_info.append(info)
 
-    with open(out_file, "w") as out_handle:
-        f_tsv = csv.DictWriter(out_handle, headers, delimiter="\t")
-        f_tsv.writeheader()
-        f_tsv.writerows(mapping_info)
+    mapping_info_df = pd.DataFrame(mapping_info)
+    if "output" in kwargs:
+        mapping_info_df.to_csv(kwargs["output"], sep="\t", index=False)
+    return mapping_info_df
 
 
 def main():
@@ -122,10 +126,10 @@ def main():
     args = parser.parse_args()
     if args.statlist:
         method = 1
-        flagstats_summary(args.statlist, args.outfile, method)
+        flagstats_summary(args.statlist, method, output=args.outfile)
     if args.statfiles:
         method = 2
-        flagstats_summary(args.statfiles, args.outfile, method)
+        flagstats_summary(args.statfiles, method, output=args.outfile)
 
 
 if __name__ == "__main__":
