@@ -1,11 +1,31 @@
 #!/usr/bin/env python3
 
-import pandas as pd
-import concurrent.futures
-import re
 import os
-import sys
 import argparse
+import concurrent.futures
+import subprocess
+import pandas as pd
+import numpy as np
+from natsort import index_natsorted
+
+
+def checkm_prepare(gene_table, batch_num, bins_dir):
+    os.makedirs(bins_dir, exist_ok=True)
+
+    table_df = pd.read_csv(gene_table, sep="\t")
+    table_df = table_df.sort_values(by="bin_id",
+                                    key=lambda x: np.argsort(index_natsorted(table_df["bin_id"])))\
+                       .reset_index(drop=True)
+
+    batchid = -1
+    if len(table_df) > 0:
+        for batch in range(0, len(table_df), batch_num):
+            batchid += 1
+            table_split = table_df.iloc[batch:batch+batch_num, ]
+            table_split.to_csv(os.path.join(bins_dir, f"bins_input.{batchid}.tsv"),
+                               sep="\t", index=False, header=None)
+    else:
+        subprocess.run(f'''touch {os.path.join(bins_dir, "bins_input.0.tsv")}''', shell=True)
 
 
 def MIMAG_quality_level(row):
@@ -71,18 +91,23 @@ def checkm_reporter(checkm_list, output, threads):
         }
     )
 
-    df_["MIMAG_quality_level"] = df_.apply(lambda x: MIMAG_quality_level(x), axis=1)
-    df_["SGB_quality_level"] = df_.apply(lambda x: SGB_quality_level(x), axis=1)
+    df_["MIMAG_quality_level"] = df_.apply(
+        lambda x: MIMAG_quality_level(x), axis=1)
+    df_["SGB_quality_level"] = df_.apply(
+        lambda x: SGB_quality_level(x), axis=1)
     df_["quality_score"] = df_.apply(lambda x: quality_score(x), axis=1)
 
-    df_.to_csv(output, sep="\t", index=False)
+    if output is not None:
+        df_.to_csv(output, sep="\t", index=False)
+
     return df_
 
 
 def main():
     parser = argparse.ArgumentParser("CheckM reporter")
     parser.add_argument("--checkm_list", type=str, help="checkm out list")
-    parser.add_argument("--output", type=str, required=True, help="checkm output file")
+    parser.add_argument("--output", type=str, required=True,
+                        help="checkm output file")
     parser.add_argument(
         "--threads", type=int, default=8, help="threads used on combine CheckM output"
     )
