@@ -29,10 +29,11 @@ WORKFLOWS_MAG = [
     "rmhost_bowtie2_all",
     "rmhost_minimap2_all",
     "rmhost_kraken2_all",
+    "rmhost_kneaddata_all",
     "rmhost_report_all",
     "rmhost_all",
     "qcreport_all",
-    "assebmly_megahit_all",
+    "assembly_megahit_all",
     "assembly_idba_ud_all",
     "assembly_metaspades_all",
     "assembly_spades_all",
@@ -40,16 +41,9 @@ WORKFLOWS_MAG = [
     "assembly_opera_ms_all",
     "assembly_metaquast_all",
     "assembly_report_all",
-    "single_assembly_all",
-    "coassembly_megahit_all",
-    "coassembly_all",
     "assembly_all",
     "alignment_base_depth_all",
     "alignment_report_all",
-    "single_alignment_all",
-    "coalignment_base_depth_all",
-    "coalignment_report_all",
-    "coalignment_all",
     "alignment_all",
     "binning_metabat2_coverage_all",
     "binning_metabat2_all",
@@ -60,53 +54,18 @@ WORKFLOWS_MAG = [
     "binning_vamb_prepare_all",
     "binning_vamb_all",
     "binning_report_all",
-    "single_binning_all",
-    "cobinning_metabat2_coverage_all",
-    "cobinning_metabat2_all",
-    "cobinning_maxbin2_all",
-    "cobinning_concoct_all",
-    "cobinning_graphbin2_all",
-    "cobinning_dastools_all",
-    "cobinning_report_all",
-    "cobinning_all",
     "binning_all",
     "predict_scaftigs_gene_prodigal_all",
     "predict_scaftigs_gene_prokka_all",
     "predict_bins_gene_prodigal_all",
     "predict_bins_gene_prokka_all",
-    "single_predict_scaftigs_gene_all",
-    "single_predict_bins_gene_all",
-    "copredict_scaftigs_gene_prodigal_all",
-    "copredict_scaftigs_gene_prokka_all",
-    "copredict_bins_gene_prodigal_all",
-    "copredict_bins_gene_prokka_all",
-    "copredict_scafitgs_gene_all",
-    "copredict_bins_gene_all",
     "predict_scaftigs_gene_all",
     "predict_bins_gene_all",
-    "copredict_all",
     "predict_all",
-    "single_checkm_all",
-    "cocheckm_all",
     "checkm_all",
     "dereplicate_mags_drep_all",
     "dereplicate_mags_all",
-    "classify_short_reads_kraken2_all",
-    "single_classify_hmq_bins_gtdbtk_all",
-    "coclassify_hmq_bins_gtdbtk_all",
-    "classify_hmq_bins_gtdbtk_all",
-    "single_classify_all",
-    "coclassify_all",
     "classify_all",
-    "profiling_bgi_soap_all",
-    "profiling_bowtie2_all",
-    "profiling_metaphlan2_all",
-    "profiling_metaphlan3_all",
-    "profiling_jgi_all",
-    "profiling_bracken_all",
-    "profiling_humann2_all",
-    "profiling_humann3_all",
-    "profiling_all",
     "upload_sequencing_all",
     "upload_assembly_all",
     "upload_all",
@@ -138,7 +97,6 @@ WORKFLOWS_GENE = [
     "assembly_plass_all",
     "assembly_metaquast_all",
     "assembly_report_all",
-    "single_assembly_all",
     "predict_scaftigs_gene_prodigal_all",
     "predict_scaftigs_gene_prokka_all",
     "predict_scafitgs_gene_all",
@@ -169,8 +127,12 @@ def run_snakemake(args, unknown, snakefile, workflow):
         str(args.cores),
     ] + unknown
 
-    if args.conda_create_envs_only:
+    if "--touch" in unknown:
+        pass
+    elif args.conda_create_envs_only:
         cmd += ["--use-conda", "--conda-create-envs-only"]
+        if args.conda_prefix is not None:
+            cmd += ["--conda-prefix", args.conda_prefix]
     else:
         cmd += [
             "--rerun-incomplete",
@@ -178,7 +140,7 @@ def run_snakemake(args, unknown, snakefile, workflow):
             "--printshellcmds",
             "--reason",
             "--until",
-            args.task,
+            args.task
         ]
 
         if args.use_conda:
@@ -188,27 +150,21 @@ def run_snakemake(args, unknown, snakefile, workflow):
 
         if args.list:
             cmd += ["--list"]
-        elif args.run:
-            cmd += [""]
+        elif args.run_local:
+            cmd += ["--local-cores", str(args.local_cores),
+                    "--jobs", str(args.jobs)]
+        elif args.run_remote:
+            cmd += ["--profile", args.profile,
+                    "--local-cores", str(args.local_cores),
+                    "--jobs", str(args.jobs)]
         elif args.debug:
-            cmd += ["--debug-dag", "--dry-run"]
-        elif args.dry_run:
+            cmd += ["--debug-dag"]
+        else: 
             cmd += ["--dry-run"]
-        elif args.qsub:
-            cmd += [
-                "--cluster-config",
-                args.cluster,
-                "--jobs",
-                str(args.jobs),
-                "--latency-wait",
-                str(args.wait),
-                '--cluster "qsub -S /bin/bash -cwd \
-                -q {cluster.queue} -P {cluster.project} \
-                -l vf={cluster.mem},p={cluster.cores} \
-                -binding linear:{cluster.cores} \
-                -o {cluster.output} -e {cluster.error}"',
-            ]
 
+        if args.dry_run and ("--dry-run" not in cmd):
+            cmd += ["--dry-run"]
+ 
     cmd_str = " ".join(cmd).strip()
     print("Running metapi %s:\n%s" % (workflow, cmd_str))
 
@@ -222,100 +178,88 @@ def run_snakemake(args, unknown, snakefile, workflow):
     )
     proc.communicate()
 
+    print(f'''\nReal running cmd:\n{cmd_str}''')
+
+
+def update_config_tools(conf, begin, trimmer, rmhoster, assemblers, binners):
+    conf["params"]["simulate"]["do"] = False
+    conf["params"]["begin"] = begin
+
+    for trimmer_ in ["oas1", "sickle", "fastp"]:
+        if trimmer_ == trimmer:
+            conf["params"]["trimming"][trimmer_]["do"] = True
+        else:
+            conf["params"]["trimming"][trimmer_]["do"] = False
+
+    for rmhoster_ in ["soap", "bwa", "bowtie2", "minimap2", "kraken2", "kneaddata"]:
+        if rmhoster_ == rmhoster:
+            conf["params"]["rmhost"][rmhoster_]["do"] = True
+        else:
+            conf["params"]["rmhost"][rmhoster_]["do"] = False
+
+    for assembler_ in ["megahit", "idba_ud", "metaspades", "spades", "plass", "opera_ms"]:
+        if assembler_ in assemblers:
+            conf["params"]["assembly"][assembler_]["do"] = True
+        else:
+            conf["params"]["assembly"][assembler_]["do"] = False
+
+    for binner_ in ["metabat2", "maxbin2", "concoct", "graphbin2", "vamb", "dastools"]:
+        if binner_ in binners:
+            conf["params"]["binning"][binner_]["do"] = True
+        else:
+            conf["params"]["binning"][binner_]["do"] = False
+
+    if begin == "simulate":
+        conf["params"]["simulate"]["do"] = True
+    elif begin == "rmhost":
+        conf["params"]["trimming"][trimmer]["do"] = False
+    elif (begin == "assembly") or (begin == "binning"):
+        conf["params"]["raw"]["save_reads"] = True
+        conf["params"]["raw"]["fastqc"]["do"] = False
+        conf["params"]["qcreport"]["do"] = False
+
+        conf["params"]["trimming"][trimmer]["do"] = False
+        conf["params"]["rmhost"][rmhoster]["do"] = False
+    return conf
+
 
 def init(args, unknown):
     if args.workdir:
         project = metapi.metaconfig(args.workdir)
         print(project.__str__())
         project.create_dirs()
-        conf, cluster = project.get_config()
+        conf = project.get_config()
 
         for env_name in conf["envs"]:
             conf["envs"][env_name] = os.path.join(
                 os.path.realpath(args.workdir), f"envs/{env_name}.yaml"
             )
 
-        conf["params"]["trimming"][args.trimmer]["do"] = True
-        conf["params"]["rmhost"][args.rmhoster]["do"] = True
-
-        for assembler in args.assembler:
-            conf["params"]["assembly"][assembler]["do"] = True
-
-        for binner in args.binner:
-            conf["params"]["binning"][binner]["do"] = True
-
-        if args.begin:
-            conf["params"]["begin"] = args.begin
-            if args.begin == "simulate":
-                conf["params"]["simulate"]["do"] = True
-            elif args.begin == "trimming":
-                conf["params"]["simulate"]["do"] = False
-            elif args.begin == "rmhost":
-                conf["params"]["simulate"]["do"] = False
-                conf["params"]["trimming"]["oas1"]["do"] = False
-                conf["params"]["trimming"]["sickle"]["do"] = False
-                conf["params"]["trimming"]["fastp"]["do"] = False
-            elif args.begin == "assembly":
-                conf["params"]["simulate"]["do"] = False
-                conf["params"]["raw"]["save_reads"] = True
-                conf["params"]["raw"]["fastqc"]["do"] = False
-                conf["params"]["qcreport"]["do"] = False
-                conf["params"]["trimming"]["oas1"]["do"] = False
-                conf["params"]["trimming"]["sickle"]["do"] = False
-                conf["params"]["trimming"]["fastp"]["do"] = False
-                conf["params"]["rmhost"]["bwa"]["do"] = False
-                conf["params"]["rmhost"]["bowtie2"]["do"] = False
-                conf["params"]["rmhost"]["soap"]["do"] = False
-                conf["params"]["rmhost"]["minimap2"]["do"] = False
-            elif args.begin == "binning":
-                conf["params"]["simulate"]["do"] = False
-                conf["params"]["raw"]["save_reads"] = True
-                conf["params"]["raw"]["fastqc"]["do"] = False
-                conf["params"]["qcreport"]["do"] = False
-                conf["params"]["trimming"]["oas1"]["do"] = False
-                conf["params"]["trimming"]["sickle"]["do"] = False
-                conf["params"]["trimming"]["fastp"]["do"] = False
-                conf["params"]["rmhost"]["bwa"]["do"] = False
-                conf["params"]["rmhost"]["bowtie2"]["do"] = False
-                conf["params"]["rmhost"]["soap"]["do"] = False
-                conf["params"]["rmhost"]["minimap2"]["do"] = False
-                conf["params"]["assembly"]["idba_ud"]["do"] = False
-                conf["params"]["assembly"]["megahit"]["do"] = False
-                conf["params"]["assembly"]["metaspades"]["do"] = False
-                conf["params"]["assembly"]["spades"]["do"] = False
-                conf["params"]["assembly"]["opera_ms"]["do"] = False
+        conf = update_config_tools(
+            conf, args.begin, args.trimmer, args.rmhoster, args.assembler, args.binner
+        )
 
         if args.samples:
             conf["params"]["samples"] = args.samples
-
-        if conf["params"]["binning"]["vamb"]["cuda"]:
-            cluster["binning_vamb"]["project"] = "P18Z10200N0127_gpu"
-            cluster["binning_vamb"]["queue"] = "st_gpu1.q"
-            print("When do binning using vamb and cuda, please use gpu queue")
         else:
-            cluster["binning_vamb"]["project"] = "P18Z10200N0127"
-            cluster["binning_vamb"]["queue"] = "st.q"
+            print("Please supply samples table")
+            sys.exit(-1)
 
         metapi.update_config(
             project.config_file, project.new_config_file, conf, remove=False
         )
-        metapi.update_config(
-            project.cluster_file, project.new_cluster_file, cluster, remove=False
-        )
     else:
         print("Please supply a workdir!")
         sys.exit(-1)
 
 
 def mag_wf(args, unknown):
-    snakefile = os.path.join(os.path.dirname(
-        __file__), "snakefiles/mag_wf.smk")
+    snakefile = os.path.join(os.path.dirname(__file__), "snakefiles/mag_wf.smk")
     run_snakemake(args, unknown, snakefile, "mag_wf")
 
 
 def gene_wf(args, unknown):
-    snakefile = os.path.join(os.path.dirname(
-        __file__), "snakefiles/gene_wf.smk")
+    snakefile = os.path.join(os.path.dirname(__file__), "snakefiles/gene_wf.smk")
     run_snakemake(args, unknown, snakefile, "gene_wf")
 
 
@@ -330,8 +274,7 @@ def snakemake_summary(snakefile, configfile, task):
         task,
         "--summary",
     ]
-    cmd_out = subprocess.Popen(
-        cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+    cmd_out = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
     summary = pd.read_csv(StringIO(cmd_out.stdout.read().decode()), sep="\t")
     return summary
 
@@ -341,25 +284,32 @@ def sync(args, unknown):
         os.path.dirname(__file__), f"snakefiles/{args.workflow}.smk"
     )
     summary_df = snakemake_summary(snakefile, args.config, args.task)
-
     conf = metapi.parse_yaml(args.config)
+
     if conf["params"]["simulate"]["do"]:
-        samples_df = metapi.parse_genomes(conf)
+        samples_df = metapi.parse_genomes(
+            conf["params"]["samples"], conf["output"]["simulate"], args.check_samples
+        )
     else:
-        samples_df = metapi.parse_samples(conf)
+        samples_df = metapi.parse_samples(
+            conf["params"]["samples"],
+            conf["params"]["interleaved"],
+            conf["params"]["reads_layout"],
+            conf["params"]["begin"],
+            args.check_samples,
+        )
 
     samples_index = samples_df.index.unique()
     count = -1
     for i in range(0, len(samples_index), args.split_num):
         count += 1
-        outdir = os.path.abspath(os.path.join(
-            args.outdir, args.name + f"_{count}"))
+        outdir = os.path.abspath(os.path.join(args.outdir, args.name + f"_{count}"))
         os.makedirs(outdir, exist_ok=True)
         samples_file = os.path.join(outdir, f"samples_{count}.tsv")
         config_file = os.path.join(outdir, "config.yaml")
 
         samples = samples_df.loc[
-            samples_index[i: i + args.split_num],
+            samples_index[i : i + args.split_num],
         ]
         samples.to_csv(samples_file, sep="\t", index=False)
         conf["params"]["samples"] = samples_file
@@ -385,8 +335,7 @@ def sync(args, unknown):
                         f"rsync --archive --relative --progress {log_file_path} {outdir}\n"
                     )
 
-    print(
-        f"please change current directory to {args.workdir} to run sync script")
+    print(f"please change current directory to {args.workdir} to run sync script")
 
 
 def main():
@@ -427,6 +376,13 @@ def main():
         default="./",
         help="project workdir",
     )
+    common_parser.add_argument(
+        "--check-samples",
+        dest="check_samples",
+        default=False,
+        action="store_true",
+        help="check samples, default: False",
+    )
 
     run_parser = argparse.ArgumentParser(add_help=False)
     run_parser.add_argument(
@@ -436,29 +392,33 @@ def main():
         help="config.yaml",
     )
     run_parser.add_argument(
-        "--cluster",
+        "--profile",
         type=str,
-        default="./cluster.yaml",
-        help="cluster.yaml",
+        default="./profiles/slurm",
+        help="cluster profile name",
     )
     run_parser.add_argument(
-        "--cores", type=int, default=8, help="CPU cores"
-    )
+        "--cores",
+        type=int,
+        default=240,
+        help="all job cores, available on '--run-local'")
     run_parser.add_argument(
-        "--jobs", type=int, default=80, help="qsub job numbers"
-    )
+        "--local-cores",
+        type=int,
+        dest="local_cores",
+        default=8,
+        help="local job cores, available on '--run-remote'")
+    run_parser.add_argument(
+        "--jobs",
+        type=int,
+        default=30,
+        help="cluster job numbers, available on '--run-remote'")
     run_parser.add_argument(
         "--list",
         default=False,
         action="store_true",
         help="list pipeline rules",
     )
-    run_parser.add_argument(
-        "--run",
-        default=False,
-        action="store_true",
-        help="run pipeline",
-    )
     run_parser.add_argument(
         "--debug",
         default=False,
@@ -473,14 +433,24 @@ def main():
         help="dry run pipeline",
     )
     run_parser.add_argument(
-        "--qsub",
+        "--run-local",
         default=False,
         action="store_true",
-        help="qsub pipeline",
+        help="run pipeline on local computer",
     )
     run_parser.add_argument(
-        "--wait", type=int, default=60, help="wait given seconds"
+        "--run-remote",
+        default=False,
+        action="store_true",
+        help="run pipeline on remote cluster",
     )
+    run_parser.add_argument(
+        "--cluster-engine",
+        default="slurm",
+        choices=["slurm", "sge", "lsf", "pbs-torque"],
+        help="cluster workflow manager engine, support slurm(sbatch) and sge(qsub)"
+    )
+    run_parser.add_argument("--wait", type=int, default=60, help="wait given seconds")
     run_parser.add_argument(
         "--use-conda",
         default=False,
@@ -490,9 +460,9 @@ def main():
     )
     run_parser.add_argument(
         "--conda-prefix",
-        default="/ldfssz1/ST_META/share/User/zhujie/.conda/envs",
+        default="~/.conda/envs",
         dest="conda_prefix",
-        help="conda environment prefix"
+        help="conda environment prefix",
     )
     run_parser.add_argument(
         "--conda-create-envs-only",
@@ -502,8 +472,7 @@ def main():
         help="conda create environments only",
     )
 
-    subparsers = parser.add_subparsers(
-        title="available subcommands", metavar="")
+    subparsers = parser.add_subparsers(title="available subcommands", metavar="")
     parser_init = subparsers.add_parser(
         "init",
         formatter_class=metapi.custom_help_formatter,
@@ -525,6 +494,7 @@ def main():
         prog="metapi gene_wf",
         help="metagenome-assembly-gene pipeline",
     )
+
     parser_sync = subparsers.add_parser(
         "sync",
         formatter_class=metapi.custom_help_formatter,
@@ -543,9 +513,9 @@ samples list, tsv format required.
 
 if begin from trimming, rmhost, or assembly:
     if it is fastq:
-        the header is: [id, fq1, fq2]
+        the header is: [sample_id, assembly_group, binning_group, fq1, fq2]
     if it is sra:
-        the header is: [id, sra]
+        the header is: [sample_id, assembly_group, binning_group, sra]
 
 if begin from simulate:
         the header is: [id, genome, abundance, reads_num, model]
@@ -557,7 +527,7 @@ if begin from simulate:
         "--begin",
         type=str,
         default="trimming",
-        choices=["simulate", "trimming", "rmhost", "assembly", "binning"],
+        choices=["simulate", "trimming", "rmhost", "assembly", "binning", "checkm"],
         help="pipeline starting point",
     )
     parser_init.add_argument(
@@ -566,15 +536,15 @@ if begin from simulate:
         default="fastp",
         required=False,
         choices=["oas1", "sickle", "fastp"],
-        help="which trimmer used"
+        help="which trimmer used",
     )
     parser_init.add_argument(
         "--rmhoster",
         type=str,
         default="bowtie2",
         required=False,
-        choices=["soap", "bwa", "bowtie2", "minimap2"],
-        help="which rmhoster used"
+        choices=["soap", "bwa", "bowtie2", "minimap2", "kraken2", "kneaddata"],
+        help="which rmhoster used",
     )
     parser_init.add_argument(
         "--assembler",
@@ -582,15 +552,14 @@ if begin from simulate:
         default=["metaspades"],
         required=False,
         choices=["idba-ud", "megahit", "metaspades", "spades", "opera-ms"],
-        help="which assembler used, required when begin with binning, can be changed in config.yaml"
+        help="which assembler used, required when begin with binning, can be changed in config.yaml",
     )
     parser_init.add_argument(
         "--binner",
         nargs="+",
         required=False,
-        default=["metabat2", "concoct", "maxbin2",
-                 "graphbin2", "vamb", "dastools"],
-        help="wchich binner used"
+        default=["metabat2", "concoct", "maxbin2", "vamb", "dastools"],
+        help="wchich binner used",
     )
     parser_init.set_defaults(func=init)
 
@@ -601,8 +570,7 @@ if begin from simulate:
         type=str,
         default="all",
         choices=WORKFLOWS_MAG,
-        help="pipeline end point. Allowed values are " +
-        ", ".join(WORKFLOWS_MAG),
+        help="pipeline end point. Allowed values are " + ", ".join(WORKFLOWS_MAG),
     )
     parser_mag_wf.set_defaults(func=mag_wf)
 
@@ -613,8 +581,7 @@ if begin from simulate:
         type=str,
         default="all",
         choices=WORKFLOWS_GENE,
-        help="pipeline end point. Allowed values are " +
-        ", ".join(WORKFLOWS_GENE),
+        help="pipeline end point. Allowed values are " + ", ".join(WORKFLOWS_GENE),
     )
     parser_gene_wf.set_defaults(func=gene_wf)
 
@@ -624,11 +591,11 @@ if begin from simulate:
         nargs="?",
         type=str,
         default="mag_wf",
-        choices=["mag_wf", "gene_wf"],
-        help="workflow. Allowed values are mag_wf and gene_wf",
+        choices=["mag_wf", "gene_wf", "phage_wf"],
+        help="workflow. Allowed values are mag_wf, gene_wf and phage_wf",
     )
     parser_sync.add_argument(
-        "task",
+        "task", 
         metavar="TASK",
         nargs="?",
         type=str,
@@ -641,8 +608,7 @@ if begin from simulate:
         default="./config.yaml",
         help="config.yaml",
     )
-    parser_sync.add_argument(
-        "--name", type=str, required=True, help="project basename")
+    parser_sync.add_argument("--name", type=str, required=True, help="project basename")
     parser_sync.add_argument(
         "--outdir", type=str, required=True, help="sync to a directory"
     )
